"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class DigitalWalletDomainTypedDict(TypedDict):
    domain_name: str
    r"""The domain to add or remove."""


class DigitalWalletDomain(BaseModel):
    domain_name: str
    r"""The domain to add or remove."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, UnrecognizedStr
from gr4vy.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


SubscriptionFrequencyUnit = Union[
    Literal[
        "DAILY",
        "WEEKLY",
        "MONTHLY",
        "QUARTERLY",
        "SEMI_ANNUAL",
        "ANNUAL",
    ],
    UnrecognizedStr,
]
r"""Indicates the frequency unit for the subscription. Allowed values are: `DAILY`, `WEEKLY`, `MONTHLY`, `QUARTERLY`, `SEMI_ANNUAL`, and `ANNUAL`."""


class DlocalUPIRecurringInfoOptionsTypedDict(TypedDict):
    subscription_frequency_unit: SubscriptionFrequencyUnit
    r"""Indicates the frequency unit for the subscription. Allowed values are: `DAILY`, `WEEKLY`, `MONTHLY`, `QUARTERLY`, `SEMI_ANNUAL`, and `ANNUAL`."""
    subscription_frequency: int
    r"""Indicates the frequency for the subscription."""
    subscription_start_at: str
    r"""Indicates the start date for the subscription in format `YYYYMMDD`."""
    subscription_end_at: str
    r"""Indicates the end date for the subscription in format `YYYYMMDD`."""


class DlocalUPIRecurringInfoOptions(BaseModel):
    subscription_frequency_unit: Annotated[
        SubscriptionFrequencyUnit, PlainValidator(validate_open_enum(False))
    ]
    r"""Indicates the frequency unit for the subscription. Allowed values are: `DAILY`, `WEEKLY`, `MONTHLY`, `QUARTERLY`, `SEMI_ANNUAL`, and `ANNUAL`."""

    subscription_frequency: int
    r"""Indicates the frequency for the subscription."""

    subscription_start_at: str
    r"""Indicates the start date for the subscription in format `YYYYMMDD`."""

    subscription_end_at: str
    r"""Indicates the end date for the subscription in format `YYYYMMDD`."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .auditlogaction import AuditLogAction
from .auditlogentryresource import AuditLogEntryResource, AuditLogEntryResourceTypedDict
from .auditlogentryuser import AuditLogEntryUser, AuditLogEntryUserTypedDict
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AuditLogEntryTypedDict(TypedDict):
    resource: AuditLogEntryResourceTypedDict
    action: AuditLogAction
    user: AuditLogEntryUserTypedDict
    timestamp: datetime
    r"""The date and time that the action was performed."""
    type: Literal["audit-log"]
    r"""Always `audit-log`."""
    id: NotRequired[Nullable[str]]
    r"""The ID for the audit log entry."""
    merchant_account_id: NotRequired[Nullable[str]]
    r"""The ID of the merchant account this entry was created for."""


class AuditLogEntry(BaseModel):
    resource: AuditLogEntryResource

    action: Annotated[AuditLogAction, PlainValidator(validate_open_enum(False))]

    user: AuditLogEntryUser

    timestamp: datetime
    r"""The date and time that the action was performed."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["audit-log"]], AfterValidator(validate_const("audit-log"))
        ],
        pydantic.Field(alias="type"),
    ] = "audit-log"
    r"""Always `audit-log`."""

    id: OptionalNullable[str] = UNSET
    r"""The ID for the audit log entry."""

    merchant_account_id: OptionalNullable[str] = UNSET
    r"""The ID of the merchant account this entry was created for."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "id", "merchant_account_id"]
        nullable_fields = ["id", "merchant_account_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from enum import Enum
from typing import Any, Optional


def get_discriminator(model: Any, fieldname: str, key: str) -> str:
    """
    Recursively search for the discriminator attribute in a model.

    Args:
        model (Any): The model to search within.
        fieldname (str): The name of the field to search for.
        key (str): The key to search for in dictionaries.

    Returns:
        str: The name of the discriminator attribute.

    Raises:
        ValueError: If the discriminator attribute is not found.
    """
    upper_fieldname = fieldname.upper()

    def get_field_discriminator(field: Any) -> Optional[str]:
        """Search for the discriminator attribute in a given field."""

        if isinstance(field, dict):
            if key in field:
                return f"{field[key]}"

        if hasattr(field, fieldname):
            attr = getattr(field, fieldname)
            if isinstance(attr, Enum):
                return f"{attr.value}"
            return f"{attr}"

        if hasattr(field, upper_fieldname):
            attr = getattr(field, upper_fieldname)
            if isinstance(attr, Enum):
                return f"{attr.value}"
            return f"{attr}"

        return None

    def search_nested_discriminator(obj: Any) -> Optional[str]:
        """Recursively search for discriminator in nested structures."""
        # First try direct field lookup
        discriminator = get_field_discriminator(obj)
        if discriminator is not None:
            return discriminator

        # If it's a dict, search in nested values
        if isinstance(obj, dict):
            for value in obj.values():
                if isinstance(value, list):
                    # Search in list items
                    for item in value:
                        nested_discriminator = search_nested_discriminator(item)
                        if nested_discriminator is not None:
                            return nested_discriminator
                elif isinstance(value, dict):
                    # Search in nested dict
                    nested_discriminator = search_nested_discriminator(value)
                    if nested_discriminator is not None:
                        return nested_discriminator

        return None

    if isinstance(model, list):
        for field in model:
            discriminator = search_nested_discriminator(field)
            if discriminator is not None:
                return discriminator

    discriminator = search_nested_discriminator(model)
    if discriminator is not None:
        return discriminator

    raise ValueError(f"Could not find discriminator field {fieldname} in {model}")



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .airline import Airline, AirlineTypedDict
from .cartitem import CartItem, CartItemTypedDict
from .guestbuyer_input import GuestBuyerInput, GuestBuyerInputTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class CheckoutSessionCreateTypedDict(TypedDict):
    cart_items: NotRequired[Nullable[List[CartItemTypedDict]]]
    r"""An array of cart items that represents the line items of a transaction."""
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Any additional information about the transaction that you would like to store as key-value pairs. This data is passed to payment service providers that support it."""
    buyer: NotRequired[Nullable[GuestBuyerInputTypedDict]]
    r"""Provide buyer details for the transaction. No buyer resource will be created on Gr4vy when used."""
    airline: NotRequired[Nullable[AirlineTypedDict]]
    r"""The airline addendum data which describes the airline booking associated with this transaction."""
    expires_in: NotRequired[float]


class CheckoutSessionCreate(BaseModel):
    cart_items: OptionalNullable[List[CartItem]] = UNSET
    r"""An array of cart items that represents the line items of a transaction."""

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Any additional information about the transaction that you would like to store as key-value pairs. This data is passed to payment service providers that support it."""

    buyer: OptionalNullable[GuestBuyerInput] = UNSET
    r"""Provide buyer details for the transaction. No buyer resource will be created on Gr4vy when used."""

    airline: OptionalNullable[Airline] = UNSET
    r"""The airline addendum data which describes the airline booking associated with this transaction."""

    expires_in: Optional[float] = 3600

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["cart_items", "metadata", "buyer", "airline", "expires_in"]
        nullable_fields = ["cart_items", "metadata", "buyer", "airline"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List
from typing_extensions import NotRequired, TypedDict


class PaypalOptionsTypedDict(TypedDict):
    additional_data: NotRequired[Nullable[List[Dict[str, str]]]]
    r"""Additional Set Transaction Context Values (STC) to be sent to PayPal as part of the transaction."""


class PaypalOptions(BaseModel):
    additional_data: OptionalNullable[List[Dict[str, str]]] = UNSET
    r"""Additional Set Transaction Context Values (STC) to be sent to PayPal as part of the transaction."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["additional_data"]
        nullable_fields = ["additional_data"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .giftcarderrorcode import GiftCardErrorCode
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GiftCardSummaryTypedDict(TypedDict):
    merchant_account_id: str
    r"""The ID of the merchant account this buyer belongs to."""
    bin: str
    r"""The first 6 digits of the full gift card number."""
    sub_bin: str
    r"""The 3 digits after the `bin` of the full gift card number."""
    last4: str
    r"""The last 4 digits for the gift card."""
    type: Literal["gift-card"]
    r"""Always `gift-card`."""
    id: NotRequired[Nullable[str]]
    r"""The ID for the gift card."""
    currency: NotRequired[Nullable[str]]
    r"""The ISO-4217 currency code that this gift card has a balance for."""
    expiration_date: NotRequired[Nullable[datetime]]
    r"""The date and time when this gift card expires. This is a full date/time and may be more accurate than the actual expiry date received by the gift card service."""
    balance: NotRequired[Nullable[int]]
    r"""The amount remaining on the balance for this gift card according to the gift card service. This may be `null` if the balance could not be fetched."""
    balance_error_code: NotRequired[Nullable[GiftCardErrorCode]]
    r"""If the last balance update failed, this will contain the internal code for this error."""
    balance_raw_error_code: NotRequired[Nullable[str]]
    r"""If the last balance update failed, this will contain the the raw error code received from the gift card provider."""
    balance_raw_error_message: NotRequired[Nullable[str]]
    r"""If the last balance update failed, this will contain the the raw error message received from the gift card provider."""


class GiftCardSummary(BaseModel):
    merchant_account_id: str
    r"""The ID of the merchant account this buyer belongs to."""

    bin: str
    r"""The first 6 digits of the full gift card number."""

    sub_bin: str
    r"""The 3 digits after the `bin` of the full gift card number."""

    last4: str
    r"""The last 4 digits for the gift card."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["gift-card"]], AfterValidator(validate_const("gift-card"))
        ],
        pydantic.Field(alias="type"),
    ] = "gift-card"
    r"""Always `gift-card`."""

    id: OptionalNullable[str] = UNSET
    r"""The ID for the gift card."""

    currency: OptionalNullable[str] = UNSET
    r"""The ISO-4217 currency code that this gift card has a balance for."""

    expiration_date: OptionalNullable[datetime] = UNSET
    r"""The date and time when this gift card expires. This is a full date/time and may be more accurate than the actual expiry date received by the gift card service."""

    balance: OptionalNullable[int] = UNSET
    r"""The amount remaining on the balance for this gift card according to the gift card service. This may be `null` if the balance could not be fetched."""

    balance_error_code: Annotated[
        OptionalNullable[GiftCardErrorCode], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""If the last balance update failed, this will contain the internal code for this error."""

    balance_raw_error_code: OptionalNullable[str] = UNSET
    r"""If the last balance update failed, this will contain the the raw error code received from the gift card provider."""

    balance_raw_error_message: OptionalNullable[str] = UNSET
    r"""If the last balance update failed, this will contain the the raw error message received from the gift card provider."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "id",
            "currency",
            "expiration_date",
            "balance",
            "balance_error_code",
            "balance_raw_error_code",
            "balance_raw_error_message",
        ]
        nullable_fields = [
            "id",
            "currency",
            "expiration_date",
            "balance",
            "balance_error_code",
            "balance_raw_error_code",
            "balance_raw_error_message",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ErrorLocation = Union[
    Literal[
        "query",
        "body",
        "path",
        "header",
        "unknown",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .digitalwalletprovider import DigitalWalletProvider
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Any, Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DigitalWalletTypedDict(TypedDict):
    id: str
    r"""The ID for the digital wallet."""
    merchant_account_id: str
    r"""The ID of the merchant account this digital wallet belongs to."""
    provider: DigitalWalletProvider
    merchant_name: str
    r"""The name of the merchant the digital wallet is registered to."""
    domain_names: List[str]
    r"""The list of domain names that a digital wallet can be used on (deprecated)."""
    created_at: datetime
    r"""The date this buyer was created at."""
    updated_at: datetime
    r"""The date this buyer was last updated at."""
    type: Literal["digital-wallet"]
    r"""Always `digital-wallet`."""
    merchant_display_name: NotRequired[Nullable[str]]
    r"""The consumer facing name of the merchant."""
    merchant_url: NotRequired[Nullable[str]]
    r"""The main URL of the merchant."""
    merchant_country_code: NotRequired[Nullable[str]]
    r"""The country code where the merchant is registered."""
    active_certificate_count: NotRequired[int]
    r"""The number of active custom certificates registered for this digital wallet (Apple Pay only)."""
    pending_certificate_count: NotRequired[int]
    r"""The number of pending custom certificates registered for this digital wallet (Apple Pay only)."""
    expired_certificate_count: NotRequired[int]
    r"""The number of expired custom certificates registered for this digital wallet (Apple Pay only)."""
    fields: NotRequired[Nullable[Dict[str, Any]]]
    r"""Custom attributes for some digital wallets. Currently only used by Click to Pay."""


class DigitalWallet(BaseModel):
    id: str
    r"""The ID for the digital wallet."""

    merchant_account_id: str
    r"""The ID of the merchant account this digital wallet belongs to."""

    provider: Annotated[
        DigitalWalletProvider, PlainValidator(validate_open_enum(False))
    ]

    merchant_name: str
    r"""The name of the merchant the digital wallet is registered to."""

    domain_names: List[str]
    r"""The list of domain names that a digital wallet can be used on (deprecated)."""

    created_at: datetime
    r"""The date this buyer was created at."""

    updated_at: datetime
    r"""The date this buyer was last updated at."""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["digital-wallet"]],
            AfterValidator(validate_const("digital-wallet")),
        ],
        pydantic.Field(alias="type"),
    ] = "digital-wallet"
    r"""Always `digital-wallet`."""

    merchant_display_name: OptionalNullable[str] = UNSET
    r"""The consumer facing name of the merchant."""

    merchant_url: OptionalNullable[str] = UNSET
    r"""The main URL of the merchant."""

    merchant_country_code: OptionalNullable[str] = UNSET
    r"""The country code where the merchant is registered."""

    active_certificate_count: Optional[int] = 0
    r"""The number of active custom certificates registered for this digital wallet (Apple Pay only)."""

    pending_certificate_count: Optional[int] = 0
    r"""The number of pending custom certificates registered for this digital wallet (Apple Pay only)."""

    expired_certificate_count: Optional[int] = 0
    r"""The number of expired custom certificates registered for this digital wallet (Apple Pay only)."""

    fields: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Custom attributes for some digital wallets. Currently only used by Click to Pay."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "merchant_display_name",
            "merchant_url",
            "merchant_country_code",
            "active_certificate_count",
            "pending_certificate_count",
            "expired_certificate_count",
            "fields",
        ]
        nullable_fields = [
            "merchant_display_name",
            "merchant_url",
            "merchant_country_code",
            "fields",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class WpayEverdaypayOptionsTypedDict(TypedDict):
    merchant_defined_data: NotRequired[Nullable[Dict[str, str]]]
    r"""A dictionary of merchant defined data, to be passed to Wpay for anti-fraud control."""
    customer_id: NotRequired[Nullable[str]]
    r"""The customer ID for the Everyday Rewards account."""
    rewards_access_token: NotRequired[Nullable[str]]
    r"""The access token for the Everyday Rewards account."""
    device_id: NotRequired[Nullable[str]]
    r"""The ID of the device on which the payment is occuring."""
    post_payment_redirect: NotRequired[Nullable[bool]]
    r"""Whether the transaction should redirect post-payment"""


class WpayEverdaypayOptions(BaseModel):
    merchant_defined_data: OptionalNullable[Dict[str, str]] = UNSET
    r"""A dictionary of merchant defined data, to be passed to Wpay for anti-fraud control."""

    customer_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="customerId")
    ] = UNSET
    r"""The customer ID for the Everyday Rewards account."""

    rewards_access_token: Annotated[
        OptionalNullable[str], pydantic.Field(alias="rewardsAccessToken")
    ] = UNSET
    r"""The access token for the Everyday Rewards account."""

    device_id: Annotated[OptionalNullable[str], pydantic.Field(alias="deviceId")] = (
        UNSET
    )
    r"""The ID of the device on which the payment is occuring."""

    post_payment_redirect: Annotated[
        OptionalNullable[bool], pydantic.Field(alias="postPaymentRedirect")
    ] = UNSET
    r"""Whether the transaction should redirect post-payment"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "merchant_defined_data",
            "customerId",
            "rewardsAccessToken",
            "deviceId",
            "postPaymentRedirect",
        ]
        nullable_fields = [
            "merchant_defined_data",
            "customerId",
            "rewardsAccessToken",
            "deviceId",
            "postPaymentRedirect",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


AntiFraudDecision = Union[
    Literal[
        "accept",
        "error",
        "exception",
        "reject",
        "review",
        "skipped",
        "pending",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cartitem import CartItem, CartItemTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class PaymentOptionRequestTypedDict(TypedDict):
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""The metadata to used to evaluate checkout rules, which will help determine the right payment options to display."""
    country: NotRequired[Nullable[str]]
    r"""The country code used to evaluate checkout rules, and which are used to help determine the right payment options to display."""
    currency: NotRequired[Nullable[str]]
    r"""The currency code used to evaluate checkout rules, and which are used to help determine the right payment options to display."""
    amount: NotRequired[Nullable[int]]
    r"""The amount used to evaluate checkout rules, and which are used to help determine the right payment options to display."""
    locale: NotRequired[str]
    r"""The locale used to determine the labels for each payment option."""
    cart_items: NotRequired[Nullable[List[CartItemTypedDict]]]
    r"""The cart items used to evaluate checkout rules, and which are used to help determine the right payment options to display."""


class PaymentOptionRequest(BaseModel):
    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""The metadata to used to evaluate checkout rules, which will help determine the right payment options to display."""

    country: OptionalNullable[str] = UNSET
    r"""The country code used to evaluate checkout rules, and which are used to help determine the right payment options to display."""

    currency: OptionalNullable[str] = UNSET
    r"""The currency code used to evaluate checkout rules, and which are used to help determine the right payment options to display."""

    amount: OptionalNullable[int] = UNSET
    r"""The amount used to evaluate checkout rules, and which are used to help determine the right payment options to display."""

    locale: Optional[str] = "en"
    r"""The locale used to determine the labels for each payment option."""

    cart_items: OptionalNullable[List[CartItem]] = UNSET
    r"""The cart items used to evaluate checkout rules, and which are used to help determine the right payment options to display."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "country",
            "currency",
            "amount",
            "locale",
            "cart_items",
        ]
        nullable_fields = ["metadata", "country", "currency", "amount", "cart_items"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
